/*
 *   LADCCA
 *    
 *   Copyright (C) 2002 Robert Ham (node@users.sourceforge.net)
 *    
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdint.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <stdio.h>
#include <netdb.h>
extern int h_errno;
#include <arpa/inet.h>
#include <unistd.h>
#include <ctype.h>

#include <ladcca/xmalloc.h>
#include <ladcca/debug.h>

int
cca_sendall (int socket, const void * buf, size_t buf_size, int flags)
{
  int sent, left, err;
  char * new_buf;
  size_t new_buf_size;
  uint32_t * iptr;
  
  /* create the new buffer */
  new_buf_size = buf_size + sizeof (uint32_t);
  new_buf = cca_malloc (new_buf_size);
  
  /* create the packet header */
  iptr = (uint32_t *) new_buf;
  *iptr = htonl (new_buf_size);
  
  /* check to see if the size has been truncated
   * (should never happen - we're not shifting much data around) */
  if (ntohl (*iptr) != new_buf_size)
    {
      fprintf (stderr, "%s: buf_size was truncated by htonl()!\n",
               __FUNCTION__);
      free (new_buf);
      return -1;
    }
  
  /* fill the packet */
  iptr++;
  memcpy (iptr, buf, buf_size);
  
  sent = 0;
  left = new_buf_size;
  while (sent < new_buf_size)
    {
      err = send (socket, new_buf + sent, left, flags);

      /* connection was closed */
      if (err == 0 && sent == 0)
        {
          free (new_buf);
          return (-2);
        }

      if (err == -1)
        {
          if (errno == EINTR)
            continue;
          
          fprintf (stderr, "%s: error sending data: %s\n", __FUNCTION__,
                   strerror (errno));
          free (new_buf);
          return (-1);
        }
      
      sent += err;
      left -= err;
    }

  free (new_buf);
  return sent - sizeof (uint32_t);
}

int 
cca_recvall (int socket, void ** buf_ptr, size_t * buf_size_ptr, int flags)
{
  int err, recvd, left;
  char * buf = NULL;
  size_t buf_size;
  uint32_t * iptr;
  size_t packet_size;
  char * recvd_buf;
  
  buf_size = 64;
  buf = cca_malloc (buf_size);      
  
  recvd = 0;
  while (recvd < sizeof (uint32_t))
    {
      err = recv (socket, buf + recvd, sizeof (uint32_t) - recvd, flags);
      
      /* check if the socket was closed */
      if (err == 0 && recvd == 0)
        {
          free (buf);
          return (-2);
        }
      
      if (err == -1)
        {
          if (errno == EINTR)
            continue;

          fprintf (stderr, "%s: error recieving data: %s\n", __FUNCTION__,
                   strerror (errno));
          free (buf);
          return (-1);
        }
      
      recvd += err;
    }
  
  iptr = (uint32_t *) buf;
  packet_size = ntohl (*iptr);
  if (buf_size < packet_size)
    {
      buf_size = packet_size;
      free (buf);
      buf = cca_malloc (buf_size);
    }
  
  left = packet_size - recvd;
  while (recvd < packet_size)
    {
      err = recv (socket, buf + recvd, left, flags);
      if (err == -1)
        {
          fprintf (stderr, "%s: error recieving data: %s\n", __FUNCTION__,
                   strerror (errno));
          return (-1);
        }
      
      recvd += err;
      left -= err;
    }

  /* create the buf and return it */
  packet_size -= sizeof (uint32_t);
  recvd_buf = cca_malloc (packet_size);
  iptr = (uint32_t *) buf;
  iptr++;
  memcpy (recvd_buf, iptr, packet_size);
  
  *buf_ptr = recvd_buf;
  *buf_size_ptr = packet_size;

  
  free (buf);
  return packet_size;
}


int
cca_open_socket (int * sock, const char * host, unsigned int port)
{
  struct sockaddr_in server_addr;
  struct hostent * server_ent;
  struct in_addr * server_in_addr;
  int err;
  
  /* create the socket */
  CCA_DEBUG ("creating socket");
  *sock = socket(AF_INET, SOCK_STREAM, 0); 
  if (*sock == -1)
    {
      fprintf (stderr, "%s: could not create socket: %s\n", __FUNCTION__,
               strerror (errno));
      return -1;
    }
  CCA_DEBUG ("socket created");
  
  /* lookup the server ip */
  CCA_DEBUG ("looking up hostname");
  server_ent = gethostbyname (host);
  if (!server_ent)
    {
      fprintf (stderr, "%s: could not find dns entry for '%s': %s\n",
               __FUNCTION__, host, hstrerror (h_errno));
      close (*sock);
      return 1;
    }
  server_in_addr = (struct in_addr *) server_ent->h_addr_list[0];
  CCA_DEBUGARGS ("found hostname: '%s', ip: '%s'",
                 server_ent->h_name,
                 inet_ntoa (*server_in_addr));
  
  /* make up the server addr */
  server_addr.sin_family = AF_INET;
  server_addr.sin_port = htons(port);
  server_addr.sin_addr = *server_in_addr;
  memset(&(server_addr.sin_zero), '\0', 8);
  
  /* make the connection */
  CCA_DEBUG ("connecting socket");
  err = connect (*sock,
                 (struct sockaddr *) &server_addr,
                 sizeof (struct sockaddr));
  if (err)
    {
      fprintf (stderr, "%s: error connecting to destination '%s:%d': %s\n",
               __FUNCTION__, host, port,
               strerror (errno));
      close (*sock);
      return 1;
    }
  CCA_DEBUG ("socket connected");
  
  return 0;
}                                                                


/* EOF */
